@@ -16,8 +16,10 @@ import numpy as np
 from matplotlib.lines import Line2D
 from mplcursors import cursor
 import warnings
-from reliability.Distributions import Weibull_Distribution, Normal_Distribution, Lognormal_Distribution, Exponential_Distribution, Gamma_Distribution, Beta_Distribution, Loglogistic_Distribution
+from reliability.Distributions import Weibull_Distribution, Normal_Distribution, Lognormal_Distribution, Exponential_Distribution,\
+    Gamma_Distribution, Beta_Distribution,Loglogistic_Distribution, Gumbel_Distribution
 from reliability.Fitters import Fit_Everything
+from reliability.Utils import colorprint
 from matplotlib.widgets import Slider, RadioButtons
 import random
 
@@ -32,7 +34,7 @@ class similar_distributions:
 
     Inputs:
     distribution - a distribution object created using the reliability.Distributions module
-    include_location_shifted - True/False. Default is True. When set to True it will include Weibull_3P, Lognormal_3P, Gamma_3P, Expon_2P, Loglogistic_3P
+    include_location_shifted - True/False. Default is True. When set to True it will include Weibull_3P, Lognormal_3P, Gamma_3P, Exponential_2P, Loglogistic_3P
     show_plot - True/False. Default is True
     print_results - True/False. Default is True
     number_of_distributions_to_show - the number of similar distributions to show. Default is 3. If the number specified exceeds the number available (typically 10), then the number specified will automatically be reduced.
@@ -50,7 +52,7 @@ class similar_distributions:
 
     def __init__(self, distribution, include_location_shifted=True, show_plot=True, print_results=True, number_of_distributions_to_show=3):
         # ensure the input is a distribution object
-        if type(distribution) not in [Weibull_Distribution, Normal_Distribution, Lognormal_Distribution, Exponential_Distribution, Gamma_Distribution, Beta_Distribution, Loglogistic_Distribution]:
+        if type(distribution) not in [Weibull_Distribution, Normal_Distribution, Lognormal_Distribution, Exponential_Distribution, Gamma_Distribution, Beta_Distribution, Loglogistic_Distribution, Gumbel_Distribution]:
             raise ValueError('distribution must be a probability distribution object from the reliability.Distributions module. First define the distribution using Reliability.Distributions.___')
 
         # sample the CDF from 0.001 to 0.999. These samples will be used to fit all other distributions.
@@ -65,7 +67,7 @@ class similar_distributions:
             else:
                 negative_values_error = True
         if negative_values_error is True:
-            print('WARNING: The input distribution has non-negligible area for x<0. Samples from this region have been discarded to enable other distributions to be fitted.')
+            colorprint('WARNING: The input distribution has non-negligible area for x<0. Samples from this region have been discarded to enable other distributions to be fitted.',text_color='red')
 
         fitted_results = Fit_Everything(failures=RVS_filtered, print_results=False, show_probability_plot=False, show_histogram_plot=False, show_PP_plot=False)  # fit all distributions to the filtered samples
         ranked_distributions = list(fitted_results.results.index.values)
@@ -88,31 +90,39 @@ class similar_distributions:
                 ranked_distributions_objects.append(Lognormal_Distribution(mu=fitted_results.Lognormal_2P_mu, sigma=fitted_results.Lognormal_2P_sigma))
                 ranked_distributions_labels.append(str('Lognormal_2P (μ=' + str(round(fitted_results.Lognormal_2P_mu, sigfig)) + ',σ=' + str(round(fitted_results.Lognormal_2P_sigma, sigfig)) + ')'))
             elif dist_name == 'Exponential_1P':
-                ranked_distributions_objects.append(Exponential_Distribution(Lambda=fitted_results.Expon_1P_lambda))
-                ranked_distributions_labels.append(str('Exponential_1P (lambda=' + str(round(fitted_results.Expon_1P_lambda, sigfig)) + ')'))
+                ranked_distributions_objects.append(Exponential_Distribution(Lambda=fitted_results.Exponential_1P_lambda))
+                ranked_distributions_labels.append(str('Exponential_1P (lambda=' + str(round(fitted_results.Exponential_1P_lambda, sigfig)) + ')'))
             elif dist_name == 'Beta_2P':
                 ranked_distributions_objects.append(Beta_Distribution(alpha=fitted_results.Beta_2P_alpha, beta=fitted_results.Beta_2P_beta))
                 ranked_distributions_labels.append(str('Beta_2P (α=' + str(round(fitted_results.Beta_2P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Beta_2P_beta, sigfig)) + ')'))
             elif dist_name == 'Loglogistic_2P':
                 ranked_distributions_objects.append(Loglogistic_Distribution(alpha=fitted_results.Loglogistic_2P_alpha, beta=fitted_results.Loglogistic_2P_beta))
                 ranked_distributions_labels.append(str('Loglogistic_2P (α=' + str(round(fitted_results.Loglogistic_2P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Loglogistic_2P_beta, sigfig)) + ')'))
+            elif dist_name == 'Gumbel_2P':
+                ranked_distributions_objects.append(Gumbel_Distribution(mu=fitted_results.Gumbel_2P_mu, sigma=fitted_results.Gumbel_2P_sigma))
+                ranked_distributions_labels.append(str('Gumbel_2P (μ=' + str(round(fitted_results.Gumbel_2P_mu, sigfig)) + ',σ=' + str(round(fitted_results.Gumbel_2P_sigma, sigfig)) + ')'))
 
             if include_location_shifted is True:
                 if dist_name == 'Weibull_3P':
-                    ranked_distributions_objects.append(Weibull_Distribution(alpha=fitted_results.Weibull_3P_alpha, beta=fitted_results.Weibull_3P_beta, gamma=fitted_results.Weibull_3P_gamma))
-                    ranked_distributions_labels.append(str('Weibull_3P (α=' + str(round(fitted_results.Weibull_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Weibull_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Weibull_3P_gamma, sigfig)) + ')'))
+                    if fitted_results.Weibull_3P_gamma is not 0:
+                        ranked_distributions_objects.append(Weibull_Distribution(alpha=fitted_results.Weibull_3P_alpha, beta=fitted_results.Weibull_3P_beta, gamma=fitted_results.Weibull_3P_gamma))
+                        ranked_distributions_labels.append(str('Weibull_3P (α=' + str(round(fitted_results.Weibull_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Weibull_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Weibull_3P_gamma, sigfig)) + ')'))
                 elif dist_name == 'Gamma_3P':
-                    ranked_distributions_objects.append(Gamma_Distribution(alpha=fitted_results.Gamma_3P_alpha, beta=fitted_results.Gamma_3P_beta, gamma=fitted_results.Gamma_3P_gamma))
-                    ranked_distributions_labels.append(str('Gamma_3P (α=' + str(round(fitted_results.Gamma_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Gamma_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Gamma_3P_gamma, sigfig)) + ')'))
+                    if fitted_results.Gamma_3P_gamma is not 0:
+                        ranked_distributions_objects.append(Gamma_Distribution(alpha=fitted_results.Gamma_3P_alpha, beta=fitted_results.Gamma_3P_beta, gamma=fitted_results.Gamma_3P_gamma))
+                        ranked_distributions_labels.append(str('Gamma_3P (α=' + str(round(fitted_results.Gamma_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Gamma_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Gamma_3P_gamma, sigfig)) + ')'))
                 elif dist_name == 'Lognormal_3P':
-                    ranked_distributions_objects.append(Lognormal_Distribution(mu=fitted_results.Lognormal_3P_mu, sigma=fitted_results.Lognormal_3P_sigma, gamma=fitted_results.Lognormal_3P_gamma))
-                    ranked_distributions_labels.append(str('Lognormal_3P (μ=' + str(round(fitted_results.Lognormal_3P_mu, sigfig)) + ',σ=' + str(round(fitted_results.Lognormal_3P_sigma, sigfig)) + ',γ=' + str(round(fitted_results.Lognormal_3P_gamma, sigfig)) + ')'))
+                    if fitted_results.Lognormal_3P_gamma is not 0:
+                        ranked_distributions_objects.append(Lognormal_Distribution(mu=fitted_results.Lognormal_3P_mu, sigma=fitted_results.Lognormal_3P_sigma, gamma=fitted_results.Lognormal_3P_gamma))
+                        ranked_distributions_labels.append(str('Lognormal_3P (μ=' + str(round(fitted_results.Lognormal_3P_mu, sigfig)) + ',σ=' + str(round(fitted_results.Lognormal_3P_sigma, sigfig)) + ',γ=' + str(round(fitted_results.Lognormal_3P_gamma, sigfig)) + ')'))
                 elif dist_name == 'Exponential_2P':
-                    ranked_distributions_objects.append(Exponential_Distribution(Lambda=fitted_results.Expon_1P_lambda, gamma=fitted_results.Expon_2P_gamma))
-                    ranked_distributions_labels.append(str('Exponential_1P (lambda=' + str(round(fitted_results.Expon_1P_lambda, sigfig)) + ',γ=' + str(round(fitted_results.Expon_2P_gamma, sigfig)) + ')'))
+                    if fitted_results.Exponential_2P_gamma is not 0:
+                        ranked_distributions_objects.append(Exponential_Distribution(Lambda=fitted_results.Exponential_1P_lambda, gamma=fitted_results.Exponential_2P_gamma))
+                        ranked_distributions_labels.append(str('Exponential_1P (lambda=' + str(round(fitted_results.Exponential_1P_lambda, sigfig)) + ',γ=' + str(round(fitted_results.Exponential_2P_gamma, sigfig)) + ')'))
                 elif dist_name == 'Loglogistic_3P':
-                    ranked_distributions_objects.append(Loglogistic_Distribution(alpha=fitted_results.Loglogistic_3P_alpha, beta=fitted_results.Loglogistic_3P_beta, gamma=fitted_results.Loglogistic_3P_gamma))
-                    ranked_distributions_labels.append(str('Loglogistic_3P (α=' + str(round(fitted_results.Loglogistic_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Loglogistic_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Loglogistic_3P_gamma, sigfig)) + ')'))
+                    if fitted_results.Loglogistic_3P_gamma is not 0:
+                        ranked_distributions_objects.append(Loglogistic_Distribution(alpha=fitted_results.Loglogistic_3P_alpha, beta=fitted_results.Loglogistic_3P_beta, gamma=fitted_results.Loglogistic_3P_gamma))
+                        ranked_distributions_labels.append(str('Loglogistic_3P (α=' + str(round(fitted_results.Loglogistic_3P_alpha, sigfig)) + ',β=' + str(round(fitted_results.Loglogistic_3P_beta, sigfig)) + ',γ=' + str(round(fitted_results.Loglogistic_3P_gamma, sigfig)) + ')'))
 
         number_of_distributions_fitted = len(ranked_distributions_objects)
         self.results = ranked_distributions_objects
@@ -466,7 +476,7 @@ class distribution_explorer:
         # initialise the radio button
         radio_ax = plt.axes([0.708, 0.25, 0.27, 0.28], facecolor=self.background_color)
         radio_ax.set_title('Distribution')
-        self.radio = RadioButtons(radio_ax, ('Weibull', 'Gamma', 'Normal', 'Lognormal', 'Beta', 'Exponential', 'Loglogistic'), active=0, activecolor=self.active_color)
+        self.radio = RadioButtons(radio_ax, ('Weibull', 'Gamma', 'Normal', 'Lognormal', 'Beta', 'Exponential', 'Loglogistic', 'Gumbel'), active=0, activecolor=self.active_color)
 
         # begin the interactive section
         distribution_explorer.__interactive(self, initial_run=True)
@@ -486,7 +496,7 @@ class distribution_explorer:
             try:  # clear the slider axis if it exists
                 plt.sca(self.ax2)
                 plt.cla()
-            except ValueError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
+            except AttributeError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
                 self.ax2 = plt.axes([0.1, 0.05, 0.8, 0.03], facecolor=self.background_color)
             self.s2 = Slider(self.ax2, param_names[2], valmin=0, valmax=500, valinit=dist.gamma)
         elif self.name == 'Gamma':
@@ -501,7 +511,7 @@ class distribution_explorer:
             try:  # clear the slider axis if it exists
                 plt.sca(self.ax2)
                 plt.cla()
-            except ValueError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
+            except AttributeError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
                 self.ax2 = plt.axes([0.1, 0.05, 0.8, 0.03], facecolor=self.background_color)
             self.s2 = Slider(self.ax2, param_names[2], valmin=0, valmax=500, valinit=dist.gamma)
         elif self.name == 'Loglogistic':
@@ -516,7 +526,7 @@ class distribution_explorer:
             try:  # clear the slider axis if it exists
                 plt.sca(self.ax2)
                 plt.cla()
-            except ValueError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
+            except AttributeError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
                 self.ax2 = plt.axes([0.1, 0.05, 0.8, 0.03], facecolor=self.background_color)
             self.s2 = Slider(self.ax2, param_names[2], valmin=0, valmax=500, valinit=dist.gamma)
         elif self.name == 'Lognormal':
@@ -531,7 +541,7 @@ class distribution_explorer:
             try:  # clear the slider axis if it exists
                 plt.sca(self.ax2)
                 plt.cla()
-            except ValueError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
+            except AttributeError:  # if the slider axis does no exist (because it was destroyed by a 2P distribution) then recreate it
                 self.ax2 = plt.axes([0.1, 0.05, 0.8, 0.03], facecolor=self.background_color)
             self.s2 = Slider(self.ax2, param_names[2], valmin=0, valmax=500, valinit=dist.gamma)
         elif self.name == 'Normal':
@@ -547,6 +557,19 @@ class distribution_explorer:
                 self.ax2.remove()  # this will destroy the axes
             except KeyError:
                 pass
+        elif self.name == 'Gumbel':
+            dist = Gumbel_Distribution(mu=0, sigma=10)
+            param_names = ['Mu', 'Sigma', '']
+            plt.sca(self.ax0)
+            plt.cla()
+            self.s0 = Slider(self.ax0, param_names[0], valmin=-100, valmax=100, valinit=dist.mu)
+            plt.sca(self.ax1)
+            plt.cla()
+            self.s1 = Slider(self.ax1, param_names[1], valmin=0.01, valmax=20, valinit=dist.sigma)
+            try:  # clear the slider axis if it exists
+                self.ax2.remove()  # this will destroy the axes
+            except KeyError:
+                pass
         elif self.name == 'Exponential':
             dist = Exponential_Distribution(Lambda=1, gamma=0)
             param_names = ['Lambda', 'Gamma', '']
@@ -599,6 +622,8 @@ class distribution_explorer:
             dist = Beta_Distribution(alpha=value1, beta=value2)
         elif self.name == 'Normal':
             dist = Normal_Distribution(mu=value1, sigma=value2)
+        elif self.name == 'Gumbel':
+            dist = Gumbel_Distribution(mu=value1, sigma=value2)
         elif self.name == 'Exponential':
             dist = Exponential_Distribution(Lambda=value1, gamma=value2)
         else:
