@@ -12,11 +12,11 @@ MCF_parametric - Mean Cumulative Function Parametric. Fits a parametric model to
 
 import numpy as np
 import matplotlib.pyplot as plt
-import warnings
 from scipy import integrate
 import pandas as pd
 import scipy.stats as ss
 from scipy.optimize import curve_fit
+from reliability.Utils import colorprint
 
 
 class reliability_growth:
@@ -119,7 +119,7 @@ class optimal_replacement_time:
         if cost_PM > cost_CM:
             raise ValueError('Cost_PM must be less than Cost_CM otherwise preventative maintenance should not be conducted.')
         if weibull_beta < 1:
-            warnings.warn('weibull_beta is < 1 so the hazard rate is decreasing, therefore preventative maintenance should not be conducted.')
+            colorprint('WARNING: weibull_beta is < 1 so the hazard rate is decreasing, therefore preventative maintenance should not be conducted.',text_color='red')
 
         if q == 1:  # as good as old
             alpha_multiple = 4  # just used for plot limits
@@ -576,7 +576,7 @@ class MCF_parametric:
 
     Outputs:
     If print_results is True, the model parameters will be printed along with a brief diagnosis of the long term health of the system based on the beta parameter.
-    time - this is the times (x values) from the scatter plot. This value is calculated using MCF_nonparametric.
+    times - this is the times (x values) from the scatter plot. This value is calculated using MCF_nonparametric.
     MCF - this is the MCF (y values) from the scatter plot. This value is calculated using MCF_nonparametric.
     alpha - the calculated alpha parameter from MCF = (t/alpha)^beta
     beta - the calculated beta parameter from MCF = (t/alpha)^beta
